#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

# pylint: disable=no-self-use

import base64
import hashlib
import hmac
from unittest import TestCase

from preggy import expect
from six import text_type

from libthumbor.url_signers.base64_hmac_sha1 import UrlSigner


class Base64HmacSha1UrlSignerTestCase(TestCase):
    def test_can_create_signer(self):
        signer = UrlSigner(security_key="something")
        expect(signer).to_be_instance_of(UrlSigner)
        expect(signer.security_key).to_equal("something")

    def test_can_sign_url(self):
        signer = UrlSigner(security_key="something")
        url = "10x11:12x13/-300x-300/center/middle/smart/some/image.jpg"
        expected = base64.urlsafe_b64encode(
            hmac.new(
                "something".encode(), text_type(url).encode("utf-8"), hashlib.sha1
            ).digest()
        )
        actual = signer.signature(url)
        expect(actual).to_equal(expected)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

"""URL composer to create options-based URLs for thumbor encryption."""

# pylint: disable=line-too-long,too-many-branches,too-many-locals,too-many-arguments

import hashlib
import re

from six import b

AVAILABLE_HALIGN = ["left", "center", "right"]
AVAILABLE_VALIGN = ["top", "middle", "bottom"]


def calculate_width_and_height(url_parts, options):
    """Appends width and height information to url"""
    width = options.get("width", 0)
    has_width = width
    height = options.get("height", 0)
    has_height = height

    flip = options.get("flip", False)
    flop = options.get("flop", False)

    if flip:
        width = width * -1
    if flop:
        height = height * -1

    if not has_width and not has_height:
        if flip:
            width = "-0"
        if flop:
            height = "-0"

    if width or height:
        url_parts.append("%sx%s" % (width, height))


def url_for(**options):
    """Returns the url for the specified options"""

    url_parts = get_url_parts(**options)
    image_hash = hashlib.md5(b(options["image_url"])).hexdigest()
    url_parts.append(image_hash)

    return "/".join(url_parts)


def unsafe_url(**options):
    """Returns the unsafe url with the specified options"""

    return "unsafe/%s" % plain_image_url(**options)


def plain_image_url(**options):
    url_parts = get_url_parts(**options)
    url_parts.append(options["image_url"])

    return "/".join(url_parts)


def get_url_parts(**options):
    if "image_url" not in options:
        raise ValueError("The image_url argument is mandatory.")

    url_parts = []

    if options.get("meta", False):
        url_parts.append("meta")

    trim = options.get("trim", None)
    if trim:
        bits = ["trim"]
        if not isinstance(trim, bool):
            bits.append(trim[0] if trim[0] else "")
            if trim[1]:
                bits.append(str(trim[1]))
        url_parts.append(":".join(bits))

    crop = options.get("crop", None)
    if crop:
        crop_left = crop[0][0]
        crop_top = crop[0][1]
        crop_right = crop[1][0]
        crop_bottom = crop[1][1]

        if crop_left > 0 or crop_top > 0 or crop_bottom > 0 or crop_right > 0:
            url_parts.append(
                "%sx%s:%sx%s" % (crop_left, crop_top, crop_right, crop_bottom)
            )

    calculate_fit_in(options, url_parts)
    calculate_width_and_height(url_parts, options)

    halign = options.get("halign", "center")
    valign = options.get("valign", "middle")

    if halign not in AVAILABLE_HALIGN:
        raise ValueError(
            'Only "left", "center" and "right" are'
            + " valid values for horizontal alignment."
        )
    if valign not in AVAILABLE_VALIGN:
        raise ValueError(
            'Only "top", "middle" and "bottom" are'
            + " valid values for vertical alignment."
        )

    if halign != "center":
        url_parts.append(halign)
    if valign != "middle":
        url_parts.append(valign)

    if options.get("smart", False):
        url_parts.append("smart")

    if options.get("filters", False):
        filters_string = ["filters"]
        for filter_value in options["filters"]:
            filters_string.append(filter_value)
        url_parts.append(":".join(filters_string))

    return url_parts


def calculate_fit_in(options, url_parts):
    fit_in = False
    full_fit_in = False

    if options.get("fit_in", None):
        fit_in = True
        url_parts.append("fit-in")

    if options.get("full_fit_in", None):
        full_fit_in = True
        url_parts.append("full-fit-in")

    if options.get("adaptive_fit_in", None):
        fit_in = True
        url_parts.append("adaptive-fit-in")

    if options.get("adaptive_full_fit_in", None):
        full_fit_in = True
        url_parts.append("adaptive-full-fit-in")

    if (fit_in or full_fit_in) and not (
        options.get("width", None) or options.get("height", None)
    ):
        raise ValueError(
            "When using fit-in or full-fit-in, you must specify width and/or height."
        )


class Url:

    unsafe_or_hash = r"(?:(?:(?P<unsafe>unsafe)|(?P<hash>.+?))/)?"
    debug = r"(?:(?P<debug>debug)/)?"
    meta = r"(?:(?P<meta>meta)/)?"
    trim = r"(?:(?P<trim>trim(?::(?:top-left|bottom-right))?(?::\d+)?)/)?"
    crop = r"(?:(?P<crop_left>\d+)x(?P<crop_top>\d+):(?P<crop_right>\d+)x(?P<crop_bottom>\d+)/)?"
    fit_in = r"(?:(?P<adaptive>adaptive-)?(?P<full>full-)?(?P<fit_in>fit-in)/)?"
    dimensions = r"(?:(?P<horizontal_flip>-)?(?P<width>(?:\d+|orig))?x(?P<vertical_flip>-)?(?P<height>(?:\d+|orig))?/)?"
    halign = r"(?:(?P<halign>left|right|center)/)?"
    valign = r"(?:(?P<valign>top|bottom|middle)/)?"
    smart = r"(?:(?P<smart>smart)/)?"
    filters = r"(?:filters:(?P<filters>.+?\))/)?"
    image = r"(?P<image>.+)"

    compiled_regex = None

    @classmethod
    def regex(cls, has_unsafe_or_hash=True):
        reg = ["/?"]

        if has_unsafe_or_hash:
            reg.append(cls.unsafe_or_hash)
        reg.append(cls.debug)
        reg.append(cls.meta)
        reg.append(cls.trim)
        reg.append(cls.crop)
        reg.append(cls.fit_in)
        reg.append(cls.dimensions)
        reg.append(cls.halign)
        reg.append(cls.valign)
        reg.append(cls.smart)
        reg.append(cls.filters)
        reg.append(cls.image)

        return "".join(reg)

    @classmethod
    def parse_decrypted(cls, url):
        if cls.compiled_regex:
            reg = cls.compiled_regex
        else:
            reg = cls.compiled_regex = re.compile(cls.regex(has_unsafe_or_hash=False))

        result = reg.match(url)

        if not result:
            return None

        result = result.groupdict()

        def int_or_0(value):
            return 0 if value is None else int(value)

        adaptive = (result.get("adaptive", "") or "").startswith("adaptive")
        full = (result.get("full", "") or "").startswith("full")

        values = {
            "debug": result["debug"] == "debug",
            "meta": result["meta"] == "meta",
            "trim": result["trim"],
            "crop": {
                "left": int_or_0(result["crop_left"]),
                "top": int_or_0(result["crop_top"]),
                "right": int_or_0(result["crop_right"]),
                "bottom": int_or_0(result["crop_bottom"]),
            },
            "adaptive": adaptive,
            "full": full,
            "fit_in": result["fit_in"] == "fit-in",
            "width": result["width"] == "orig" and "orig" or int_or_0(result["width"]),
            "height": result["height"] == "orig"
            and "orig"
            or int_or_0(result["height"]),
            "horizontal_flip": result["horizontal_flip"] == "-",
            "vertical_flip": result["vertical_flip"] == "-",
            "halign": result["halign"] or "center",
            "valign": result["valign"] or "middle",
            "smart": result["smart"] == "smart",
            "filters": result["filters"] or "",
            "image": "image" in result and result["image"] or None,
        }

        return values

    @classmethod  # NOQA
    def generate_options(
        cls,
        debug=False,
        width=0,
        height=0,
        smart=False,
        meta=False,
        trim=None,
        adaptive=False,
        full=False,
        fit_in=False,
        horizontal_flip=False,
        vertical_flip=False,
        halign="center",
        valign="middle",
        crop_left=None,
        crop_top=None,
        crop_right=None,
        crop_bottom=None,
        filters=None,
    ):
        url = []

        if debug:
            url.append("debug")

        if meta:
            url.append("meta")

        if trim:
            if isinstance(trim, bool):
                url.append("trim")
            else:
                url.append("trim:%s" % trim)

        crop = crop_left or crop_top or crop_right or crop_bottom
        if crop:
            url.append("%sx%s:%sx%s" % (crop_left, crop_top, crop_right, crop_bottom))

        if fit_in:
            fit_ops = []
            if adaptive:
                fit_ops.append("adaptive")
            if full:
                fit_ops.append("full")
            fit_ops.append("fit-in")
            url.append("-".join(fit_ops))

        if horizontal_flip:
            width = "-%s" % width
        if vertical_flip:
            height = "-%s" % height

        if width or height:
            url.append("%sx%s" % (width, height))

        if halign != "center":
            url.append(halign)
        if valign != "middle":
            url.append(valign)

        if smart:
            url.append("smart")

        if filters:
            url.append("filters:%s" % filters)

        return "/".join(url)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

# pylint: disable=no-self-use

"""libthumbor URL composer tests"""
from unittest import TestCase

from preggy import expect

from libthumbor.url import unsafe_url, url_for

IMAGE_URL = "my.server.com/some/path/to/image.jpg"
IMAGE_MD5 = "84996242f65a4d864aceb125e1c4c5ba"


def test_no_options_specified():
    """test_no_options_specified
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
    When
        I ask my library for an URL
    Then
        I get "84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(image_url=IMAGE_URL)

    expect(url).to_equal(IMAGE_MD5)


def test_url_raises_if_no_url():
    """test_url_raises_if_no_url
    Given
        An image URL of "" or null
    When
        I ask my library for an URL
    Then
        I get an exception that says image URL is mandatory
    """
    with expect.error_to_happen(
        ValueError, message="The image_url argument is mandatory."
    ):
        url_for()


def test_url_width_height_1():
    """test_url_width_height_1
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 300
    When
        I ask my library for an URL
    Then
        I get "300x0/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=300, image_url=IMAGE_URL)

    expect(url).to_equal("300x0/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_2():
    """test_url_width_height_2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a height of 300
    When
        I ask my library for an URL
    Then
        I get "0x300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(height=300, image_url=IMAGE_URL)

    expect(url).to_equal("0x300/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_3():
    """test_url_width_height_3
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
    When
        I ask my library for an URL
    Then
        I get "200x300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=200, height=300, image_url=IMAGE_URL)

    expect(url).to_equal("200x300/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_4():
    """test_url_width_height_4
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of orig
    When
        I ask my library for an URL
    Then
        I get "origx0/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width="orig", image_url=IMAGE_URL)

    expect(url).to_equal("origx0/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_5():
    """test_url_width_height_5
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a height of orig
    When
        I ask my library for an URL
    Then
        I get "0xorig/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(height="orig", image_url=IMAGE_URL)

    expect(url).to_equal("0xorig/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_6():
    """test_url_width_height_6
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 100
        And a height of orig
    When
        I ask my library for an URL
    Then
        I get "100xorig/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=100, height="orig", image_url=IMAGE_URL)

    expect(url).to_equal("100xorig/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_7():
    """test_url_width_height_7
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a height of 100
        And a width of orig
    When
        I ask my library for an URL
    Then
        I get "origx100/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width="orig", height=100, image_url=IMAGE_URL)

    expect(url).to_equal("origx100/84996242f65a4d864aceb125e1c4c5ba")


def test_url_width_height_8():
    """test_url_width_height_8
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a height of orig
        And a width of orig
    When
        I ask my library for an URL
    Then
        I get "origxorig/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width="orig", height="orig", image_url=IMAGE_URL)

    expect(url).to_equal("origxorig/84996242f65a4d864aceb125e1c4c5ba")


def test_smart_url():
    """test_smart_url
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
        And the smart flag
    When
        I ask my library for an URL
    Then
        I get "200x300/smart/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=200, height=300, smart=True, image_url=IMAGE_URL)

    expect(url).to_equal("200x300/smart/84996242f65a4d864aceb125e1c4c5ba")


def test_fit_in_url():
    """test_fit_in_url
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
        And the fit-in flag
    When
        I ask my library for an URL
    Then
        I get "fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=200, height=300, fit_in=True, image_url=IMAGE_URL)

    expect(url).to_equal("fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba")


def test_adaptive_fit_in_url():
    """test_adaptive_fit_in_url
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
        And the adaptive fit-in flag
    When
        I ask my library for an URL
    Then
        I get "adaptive-fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=200, height=300, adaptive_fit_in=True, image_url=IMAGE_URL)

    expect(url).to_equal("adaptive-fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba")


def test_fit_in_fails_if_no_width_supplied():
    with expect.error_to_happen(
        ValueError,
        message="When using fit-in or full-fit-in, "
        "you must specify width and/or height.",
    ):
        url_for(fit_in=True, image_url=IMAGE_URL)


def test_full_fit_in_fails_if_no_width_supplied():
    with expect.error_to_happen(
        ValueError,
        message="When using fit-in or full-fit-in, "
        "you must specify width and/or height.",
    ):
        url_for(full_fit_in=True, image_url=IMAGE_URL)


def test_adaptive_fit_in_fails_if_no_width_supplied():
    with expect.error_to_happen(
        ValueError,
        message="When using fit-in or full-fit-in, "
        "you must specify width and/or height.",
    ):
        url_for(adaptive_fit_in=True, image_url=IMAGE_URL)


def test_adaptive_full_fit_in_fails_if_no_width_supplied():
    with expect.error_to_happen(
        ValueError,
        message="When using fit-in or full-fit-in, "
        "you must specify width and/or height.",
    ):
        url_for(adaptive_full_fit_in=True, image_url=IMAGE_URL)


def test_full_fit_in_url():
    """test_full_fit_in_url
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
        And the full-fit-in flag
    When
        I ask my library for an URL
    Then
        I get "full-fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=200, height=300, full_fit_in=True, image_url=IMAGE_URL)

    expect(url).to_equal("full-fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba")


def test_adaptive_full_fit_in_url():
    """test_adaptive_full_fit_in_url
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
        And the adaptive full-fit-in flag
    When
        I ask my library for an URL
    Then
        I get "adaptive-full-fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(width=200, height=300, adaptive_full_fit_in=True, image_url=IMAGE_URL)

    expect(url).to_equal(
        "adaptive-full-fit-in/200x300/84996242f65a4d864aceb125e1c4c5ba"
    )


def test_flip_1():
    """test_flip_1
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And the flip flag
    When
        I ask my library for an URL
    Then
        I get "-0x0/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(flip=True, image_url=IMAGE_URL)

    expect(url).to_equal("-0x0/84996242f65a4d864aceb125e1c4c5ba")


def test_flip_2():
    """test_flip_2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And the flip flag
    When
        I ask my library for an URL
    Then
        I get "-200x0/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(flip=True, width=200, image_url=IMAGE_URL)

    expect(url).to_equal("-200x0/84996242f65a4d864aceb125e1c4c5ba")


def test_flop_1():
    """test_flop_1
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And the flop flag
    When
        I ask my library for an URL
    Then
        I get "0x-0/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(flop=True, image_url=IMAGE_URL)

    expect(url).to_equal("0x-0/84996242f65a4d864aceb125e1c4c5ba")


def test_flop_2():
    """test_flop_2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a height of 200
        And the flop flag
    When
        I ask my library for an URL
    Then
        I get "0x-200/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(flop=True, height=200, image_url=IMAGE_URL)

    expect(url).to_equal("0x-200/84996242f65a4d864aceb125e1c4c5ba")


def test_flip_flop():
    """test_flip_flop
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And the flip flag
        And the flop flag
    When
        I ask my library for an URL
    Then
        I get "-0x-0/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(flip=True, flop=True, image_url=IMAGE_URL)

    expect(url).to_equal("-0x-0/84996242f65a4d864aceb125e1c4c5ba")


def test_flip_flop2():
    """test_flip_flop2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a width of 200
        And a height of 300
        And the flip flag
        And the flop flag
    When
        I ask my library for an URL
    Then
        I get "-200x-300/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(flip=True, flop=True, width=200, height=300, image_url=IMAGE_URL)

    expect(url).to_equal("-200x-300/84996242f65a4d864aceb125e1c4c5ba")


def test_horizontal_alignment():
    """test_horizontal_alignment
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'left' horizontal alignment option
    When
        I ask my library for an URL
    Then
        I get "left/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(halign="left", image_url=IMAGE_URL)

    expect(url).to_equal("left/84996242f65a4d864aceb125e1c4c5ba")


def test_horizontal_alignment2():
    """test_horizontal_alignment2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'center' horizontal alignment option
    When
        I ask my library for an URL
    Then
        I get "84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(halign="center", image_url=IMAGE_URL)

    expect(url).to_equal("84996242f65a4d864aceb125e1c4c5ba")


def test_vertical_alignment():
    """test_vertical_alignment
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'top' vertical alignment option
    When
        I ask my library for an URL
    Then
        I get "top/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(valign="top", image_url=IMAGE_URL)

    expect(url).to_equal("top/84996242f65a4d864aceb125e1c4c5ba")


def test_vertical_alignment2():
    """test_vertical_alignment2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'middle' vertical alignment option
    When
        I ask my library for an URL
    Then
        I get "84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(valign="middle", image_url=IMAGE_URL)

    expect(url).to_equal("84996242f65a4d864aceb125e1c4c5ba")


def test_both_alignments():
    """test_both_alignments
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'left' horizontal alignment option
        And a 'top' vertical alignment option
    When
        I ask my library for an URL
    Then
        I get "left/top/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(halign="left", valign="top", image_url=IMAGE_URL)

    expect(url).to_equal("left/top/84996242f65a4d864aceb125e1c4c5ba")


def test_proper_haligns():
    """test_proper_haligns"""
    with expect.error_to_happen(
        ValueError,
        message=(
            'Only "left", "center" and "right"'
            " are valid values for horizontal alignment."
        ),
    ):
        url_for(halign="wrong", image_url=IMAGE_URL)


def test_proper_valigns():
    """test_proper_haligns"""
    with expect.error_to_happen(
        ValueError,
        message=(
            'Only "top", "middle" and "bottom"'
            " are valid values for vertical alignment."
        ),
    ):
        url_for(valign="wrong", image_url=IMAGE_URL)


def test_proper_meta():
    """test_proper_meta
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'meta' flag
    When
        I ask my library for an URL
    Then
        I get "meta/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(meta=True, image_url=IMAGE_URL)

    expect("meta/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_trim_standard():
    url = url_for(trim=True, image_url=IMAGE_URL)
    expect("trim/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_trim_pixel_and_tolerance():
    url = url_for(trim=("bottom-right", 15), image_url=IMAGE_URL)
    expect("trim:bottom-right:15/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_trim_pixel_only():
    url = url_for(trim=("top-left", None), image_url=IMAGE_URL)
    expect("trim:top-left/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_trim_tolerance_only():
    url = url_for(trim=(None, 15), image_url=IMAGE_URL)
    expect("trim::15/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_manual_crop_1():
    """test_manual_crop_1
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a manual crop left-top point of (10, 20)
        And a manual crop right-bottom point of (30, 40)
    When
        I ask my library for an URL
    Then
        I get "10x20:30x40/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(crop=((10, 20), (30, 40)), image_url=IMAGE_URL)

    expect("10x20:30x40/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_manual_crop_2():
    """test_manual_crop_2
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a manual crop left-top point of (0, 0)
        And a manual crop right-bottom point of (0, 0)
    When
        I ask my library for an URL
    Then
        I get "84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(crop=((0, 0), (0, 0)), image_url=IMAGE_URL)

    expect("84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


def test_smart_after_alignments():
    """test_smart_after_alignments
    Given
        An image URL of "my.server.com/some/path/to/image.jpg"
        And a 'smart' flag
        And a 'left' horizontal alignment option
    When
        I ask my library for an URL
    Then
        I get "left/smart/84996242f65a4d864aceb125e1c4c5ba" as URL
    """
    url = url_for(smart=True, halign="left", image_url=IMAGE_URL)

    expect("left/smart/84996242f65a4d864aceb125e1c4c5ba").to_equal(url)


class UnsafeUrlTestCase(TestCase):
    def test_should_return_a_valid_unsafe_url_with_no_params(self):
        expect("unsafe/%s" % IMAGE_URL).to_equal(unsafe_url(image_url=IMAGE_URL))

    def test_should_return_an_unsafe_url_with_width_and_height(self):
        expect("unsafe/100x140/%s" % IMAGE_URL).to_equal(
            unsafe_url(image_url=IMAGE_URL, width=100, height=140),
        )

    def test_should_return_an_unsafe_url_with_crop_and_smart(self):
        expect("unsafe/100x140/smart/%s" % IMAGE_URL).to_equal(
            unsafe_url(image_url=IMAGE_URL, width=100, height=140, smart=True),
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

"""libthumbor generic views tests"""

import os

import pytest
from preggy import expect

from libthumbor.crypto import CryptoURL

os.environ["DJANGO_SETTINGS_MODULE"] = "tests.testproj.testproj.settings"

try:
    from django.conf import settings
    from django.test import TestCase
    from django.http import QueryDict

    DJANGO_PRESENT = True
except ImportError:
    DJANGO_PRESENT = False


HTTP_NOT_FOUND = 404
HTTP_METHOD_NOT_ALLOWED = 405
HTTP_OK = 200
HTTP_BAD_REQUEST = 400


@pytest.mark.skip_if(not DJANGO_PRESENT, "django must be present to run this test")
class GenericViewsTestCase(TestCase):
    def setUp(self):
        self.url_query = QueryDict("", mutable=True)

    def test_without_url_param(self):
        response = self.client.get("/gen_url/")
        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)

    def test_generate_url_with_params_via_post(self):
        image_args = {"image_url": "globo.com/media/img/my_image.jpg"}

        response = self.client.post("/gen_url/", image_args)

        expect(response.status_code).to_equal(HTTP_METHOD_NOT_ALLOWED)

    def test_generate_url_with_params_via_get(self):
        crypto = CryptoURL(settings.THUMBOR_SECURITY_KEY)
        image_args = {"image_url": "globo.com/media/img/my_image.jpg"}
        self.url_query.update(image_args)

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_OK)
        expect(response.content).to_equal(
            settings.THUMBOR_SERVER + crypto.generate(**image_args).strip("/")
        )

    def test_passing_invalid_value_for_width(self):
        self.url_query.update(
            {"image_url": "globo.com/media/img/my_image.jpg", "width": 1.2}
        )

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)
        expect(str(response.content)).to_include(
            "The width value '1.2' is not an integer."
        )

    def test_passing_invalid_value_for_height(self):
        self.url_query.update(
            {"image_url": "globo.com/media/img/my_image.jpg", "height": "s"}
        )

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)
        expect(str(response.content)).to_include(
            "The height value 's' is not an integer."
        )

    def test_passing_invalid_aligns(self):
        self.url_query.update(
            {"image_url": "globo.com/media/img/my_image.jpg", "halign": "sss"}
        )

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)

    def test_passing_only_one_crop_value(self):
        self.url_query.update(
            {"image_url": "globo.com/media/img/my_image.jpg", "crop_left": 100}
        )

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)
        expect(str(response.content)).to_include("Missing values for cropping")

    def test_passing_only_one_crop_with_invalid_value(self):
        self.url_query.update(
            {
                "image_url": "globo.com/media/img/my_image.jpg",
                "crop_top": "bla",
                "crop_left": 200,
                "crop_right": "1",
                "crop_bottom": "blas",
            }
        )

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)
        expect(str(response.content)).to_include("Invalid values for cropping")

    def test_passing_various_erroneous_values(self):
        self.url_query.update(
            {
                "image_url": "globo.com/media/img/my_image.jpg",
                "crop_left": 100,
                "width": "aaa",
                "height": 123,
            }
        )

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_BAD_REQUEST)

    def test_passing_all_params(self):
        image_args = {
            "image_url": "globo.com/media/img/my_image.jpg",
            "halign": "left",
            "valign": "middle",
            "meta": True,
            "smart": True,
            "width": 400,
            "height": 400,
            "flip": True,
            "flop": True,
        }
        self.url_query.update(image_args)
        self.url_query.update(
            {"crop_top": 100, "crop_left": 100, "crop_bottom": 200, "crop_right": 200}
        )
        image_args.update({"crop": ((100, 100), (200, 200))})

        crypto = CryptoURL(settings.THUMBOR_SECURITY_KEY)

        response = self.client.get("/gen_url/?" + self.url_query.urlencode())

        expect(response.status_code).to_equal(HTTP_OK)
        expect(response.content).to_equal(
            settings.THUMBOR_SERVER + crypto.generate(**image_args).strip("/")
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

import re

from six import b, PY3

from libthumbor import CryptoURL, Url, Signer


def test_usage_new_format():
    key = "my-security-key"
    image = "s.glbimg.com/et/bb/f/original/2011/03/24/VN0JiwzmOw0b0lg.jpg"

    thumbor_signer = Signer(key)
    thumbor_url = Url.generate_options(
        width=300,
        height=200,
        smart=True,
        adaptive=False,
        fit_in=False,
        horizontal_flip=False,
        vertical_flip=False,
        halign="center",
        valign="middle",
        crop_left=0,
        crop_top=0,
        crop_right=0,
        crop_bottom=0,
        filters=[],
    )
    thumbor_url = ("%s/%s" % (thumbor_url, image)).lstrip("/")
    signature = thumbor_signer.signature(thumbor_url)
    if PY3:
        signature = signature.decode("ascii")
    thumbor_url = "/%s/%s" % (signature, thumbor_url)

    crypto = CryptoURL(key=key)
    url = crypto.generate(width=300, height=200, smart=True, image_url=image)

    assert url == thumbor_url


def test_thumbor_can_decrypt_lib_thumbor_generated_url_new_format():
    key = "my-security-key"
    image = "s.glbimg.com/et/bb/f/original/2011/03/24/VN0JiwzmOw0b0lg.jpg"
    thumbor_signer = Signer(key)

    crypto = CryptoURL(key=key)

    url = crypto.generate(width=300, height=200, smart=True, image_url=image)

    reg = "/([^/]+)/(.+)"
    (signature, url) = re.match(reg, url).groups()

    assert thumbor_signer.validate(b(signature), url)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

"""Generic view for create thumbor encrypted urls."""
import logging

from django.conf import settings
from django.http import (HttpResponse, HttpResponseBadRequest,
                         HttpResponseNotAllowed)

from libthumbor.crypto import CryptoURL

THUMBOR_SECURITY_KEY = getattr(settings, "THUMBOR_SECURITY_KEY", "my-security-key")

THUMBOR_SERVER = getattr(settings, "THUMBOR_SERVER", "http://localhost:8888/")


def generate_url(request):
    if request.method != "GET":
        return HttpResponseNotAllowed(["GET"])

    crypto = CryptoURL(THUMBOR_SECURITY_KEY)

    args = request.GET
    args = dict(zip(map(str, args.keys()), args.values()))
    error_message = None

    try:
        if "width" in args:
            args["width"] = int(args["width"])
    except ValueError:
        error_message = "The width value '%s' is not an integer." % args["width"]

    try:
        if "height" in args:
            args["height"] = int(args["height"])
    except ValueError:
        error_message = "The height value '%s' is not an integer." % args["height"]

    try:
        if (
            "crop_top" in args
            or "crop_left" in args
            or "crop_right" in args
            or "crop_bottom" in args
        ):
            args["crop"] = (
                (int(args["crop_left"]), int(args["crop_top"])),
                (int(args["crop_right"]), int(args["crop_bottom"])),
            )
    except KeyError:
        error_message = """
            Missing values for cropping.
            Expected all 'crop_left', 'crop_top',
            'crop_right', 'crop_bottom' values.
        """
    except ValueError:
        error_message = """
            Invalid values for cropping.
            Expected all 'crop_left', 'crop_top',
            'crop_right', 'crop_bottom' to be integers.
        """

    if error_message is not None:
        logging.warning(error_message)
        return HttpResponseBadRequest(error_message)

    try:
        return HttpResponse(
            THUMBOR_SERVER + crypto.generate(**args).strip("/"),
            content_type="text/plain",
        )
    except (ValueError, KeyError) as error:
        error_message = str(error)
        logging.warning(error_message)
        return HttpResponseBadRequest(error_message)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

"""libthumbor is the library used to access thumbor's images in python"""

from libthumbor.crypto import CryptoURL  # NOQA
from libthumbor.url import Url  # NOQA
from libthumbor.url_signers.base64_hmac_sha1 import UrlSigner as Signer  # NOQA



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

# pylint: disable=no-self-use

from __future__ import unicode_literals

from unittest import TestCase

from preggy import expect

from libthumbor.url_signers import BaseUrlSigner


class BaseSignerTestCase(TestCase):
    def test_can_create_signer(self):
        signer = BaseUrlSigner(security_key="something")
        expect(signer).to_be_instance_of(BaseUrlSigner)
        expect(signer.security_key).to_equal("something")

    def test_can_create_unicode_signer(self):
        signer = BaseUrlSigner(security_key="téste")
        expect(signer).to_be_instance_of(BaseUrlSigner)
        expect(signer.security_key).to_equal("téste")

    def test_can_validate_url(self):
        class TestSigner(BaseUrlSigner):
            def signature(self, url):
                return "%s+1" % url

        signer = TestSigner(security_key="téste")
        expect(
            signer.validate("http://www.test.com+1", "http://www.test.com")
        ).to_be_true()

    def test_has_abstract_method(self):
        signer = BaseUrlSigner(security_key="téste")

        with expect.error_to_happen(NotImplementedError):
            signer.signature("test-url")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

# pylint: disable=line-too-long,no-self-use

from unittest import TestCase

from preggy import expect

from libthumbor.url import Url


class UrlTestCase(TestCase):
    def setUp(self):
        Url.compiled_regex = None

    def test_can_get_regex(self):
        regex = Url.regex()

        expect(regex).to_equal(
            "/?(?:(?:(?P<unsafe>unsafe)|(?P<hash>.+?))/)?(?:(?P<debug>debug)/)?(?:(?P<meta>meta)/)?"
            "(?:(?P<trim>trim(?::(?:top-left|bottom-right))?(?::\\d+)?)/)?"
            "(?:(?P<crop_left>\\d+)x(?P<crop_top>\\d+):(?P<crop_right>\\d+)x(?P<crop_bottom>\\d+)/)?"
            "(?:(?P<adaptive>adaptive-)?(?P<full>full-)?(?P<fit_in>fit-in)/)?(?:(?P<horizontal_flip>-)?"
            "(?P<width>(?:\\d+|orig))?x(?P<vertical_flip>-)?(?P<height>(?:\\d+|orig))?/)?"
            "(?:(?P<halign>left|right|center)/)?(?:(?P<valign>top|bottom|middle)/)?"
            "(?:(?P<smart>smart)/)?(?:filters:(?P<filters>.+?\\))/)?(?P<image>.+)"
        )

    def test_can_get_regex_without_unsafe(self):
        regex = Url.regex(False)

        expect(regex).to_equal(
            "/?(?:(?P<debug>debug)/)?(?:(?P<meta>meta)/)?"
            "(?:(?P<trim>trim(?::(?:top-left|bottom-right))?(?::\\d+)?)/)?"
            "(?:(?P<crop_left>\\d+)x(?P<crop_top>\\d+):(?P<crop_right>\\d+)x(?P<crop_bottom>\\d+)/)?"
            "(?:(?P<adaptive>adaptive-)?(?P<full>full-)?(?P<fit_in>fit-in)/)?(?:(?P<horizontal_flip>-)?"
            "(?P<width>(?:\\d+|orig))?x(?P<vertical_flip>-)?(?P<height>(?:\\d+|orig))?/)?"
            "(?:(?P<halign>left|right|center)/)?(?:(?P<valign>top|bottom|middle)/)?"
            "(?:(?P<smart>smart)/)?(?:filters:(?P<filters>.+?\\))/)?(?P<image>.+)"
        )

    def test_parsing_invalid_url(self):
        expect(Url.compiled_regex).to_be_null()

        url = ""
        expect(Url.parse_decrypted(url)).to_be_null()

    def test_parsing_complete_url(self):
        url = (
            "/debug/meta/trim/300x200:400x500/adaptive-full-fit-in/-300x-400/"
            "left/top/smart/filters:brightness(100)/some/image.jpg"
        )

        expected = {
            "trim": "trim",
            "full": True,
            "halign": "left",
            "fit_in": True,
            "vertical_flip": True,
            "image": "some/image.jpg",
            "crop": {"top": 200, "right": 400, "bottom": 500, "left": 300},
            "height": 400,
            "width": 300,
            "meta": True,
            "horizontal_flip": True,
            "filters": "brightness(100)",
            "valign": "top",
            "debug": True,
            "adaptive": True,
            "smart": True,
        }

        result = Url.parse_decrypted(url)
        expect(result).not_to_be_null()
        expect(result).to_be_like(expected)

        # do it again to use compiled regex
        result = Url.parse_decrypted(url)
        expect(result).not_to_be_null()
        expect(result).to_be_like(expected)

    def test_can_generate_url(self):
        url = Url.generate_options(
            debug=True,
            width=300,
            height=200,
            smart=True,
            meta=True,
            trim=True,
            adaptive=True,
            full=True,
            fit_in=True,
            horizontal_flip=True,
            vertical_flip=True,
            halign="left",
            valign="top",
            crop_left=100,
            crop_top=100,
            crop_right=400,
            crop_bottom=400,
            filters="brightness(100)",
        )

        expect(url).to_equal(
            "debug/meta/trim/100x100:400x400/adaptive-full-fit-in/-300x-200/left/top/smart/filters:brightness(100)"
        )

    def test_can_generate_url_with_defaults(self):
        url = Url.generate_options()

        expect(url).to_be_empty()

    def test_can_generate_url_with_fitin(self):
        url = Url.generate_options(fit_in=True, adaptive=False, full=False)

        expect(url).to_equal("fit-in")

    def test_can_generate_url_with_custom_trim(self):
        url = Url.generate_options(
            debug=True,
            width=300,
            height=200,
            smart=True,
            meta=True,
            trim="300x200",
            adaptive=True,
            full=True,
            fit_in=True,
            horizontal_flip=True,
            vertical_flip=True,
            halign="left",
            valign="top",
            crop_left=100,
            crop_top=100,
            crop_right=400,
            crop_bottom=400,
            filters="brightness(100)",
        )

        expect(url).to_equal(
            "debug/meta/trim:300x200/100x100:400x400/adaptive-full-fit-in/-300x-200/left/top/smart/filters:brightness(100)"
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# libthumbor - python extension to thumbor
# http://github.com/heynemann/libthumbor

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

"""libthumbor cryptography tests"""

from unittest import TestCase

from preggy import expect
from six import ensure_text

from libthumbor.crypto import CryptoURL

IMAGE_URL = "my.server.com/some/path/to/image.jpg"
KEY = b"my-security-key"


class NewFormatUrlTestsMixin:
    def test_generated_url_1(self):
        url = self.crypto.generate(image_url=IMAGE_URL, width=300, height=200)
        expect(url).to_equal(
            "/8ammJH8D-7tXy6kU3lTvoXlhu4o=/300x200/my.server.com/some/path/to/image.jpg"
        )

    def test_generated_url_2(self):
        url = self.crypto.generate(
            image_url=IMAGE_URL, width=300, height=200, crop=((10, 10), (200, 200))
        )
        expect(url).to_equal(
            "/B35oBEIwztbc3jm7vsdqLez2C78=/10x10:200x200/300x200/"
            "my.server.com/some/path/to/image.jpg"
        )

    def test_generated_url_3(self):
        url = self.crypto.generate(
            image_url=IMAGE_URL,
            width=300,
            height=200,
            crop=((10, 10), (200, 200)),
            filters=("brightness(20)", "contrast(10)"),
        )
        expect(url).to_equal(
            "/as8U2DbUUtTMgvPF26LkjS3MocY=/10x10:200x200/300x200/"
            "filters:brightness(20):contrast(10)/my.server.com/some/path/to/image.jpg"
        )

    def test_generated_url_4(self):
        url = self.crypto.generate(
            image_url=IMAGE_URL,
            width=300,
            height=200,
            crop=((10, 10), (200, 200)),
            filters=("brightness(20)", "contrast(10)"),
        )
        expect(url).to_equal(
            "/as8U2DbUUtTMgvPF26LkjS3MocY=/10x10:200x200/300x200/"
            "filters:brightness(20):contrast(10)/my.server.com/some/path/to/image.jpg"
        )
        # making sure no internal state affects subsequent calls.
        url = self.crypto.generate(
            image_url=IMAGE_URL,
            width=300,
            height=200,
            crop=((10, 10), (200, 200)),
            filters=("brightness(20)", "contrast(10)"),
        )
        expect(url).to_equal(
            "/as8U2DbUUtTMgvPF26LkjS3MocY=/10x10:200x200/300x200/"
            "filters:brightness(20):contrast(10)/my.server.com/some/path/to/image.jpg"
        )


class NewFormatUrl(TestCase, NewFormatUrlTestsMixin):
    def setUp(self):
        self.crypto = CryptoURL(KEY)


class NewFormatUrlWithUnicodeKey(TestCase, NewFormatUrlTestsMixin):
    def setUp(self):
        self.crypto = CryptoURL(ensure_text(KEY))


class GenerateWithUnsafeTestCase(TestCase):
    def setUp(self):
        self.crypto = CryptoURL(KEY)

    def test_should_pass_unsafe_to_generate_and_get_an_unsafe_url(self):
        url = self.crypto.generate(
            image_url=IMAGE_URL, crop=((10, 20), (30, 40)), unsafe=True
        )
        expect(url.startswith("unsafe")).to_be_true()

    def test_should_not_get_an_unsafe_url_when_unsafe_is_false(self):
        url = self.crypto.generate(
            image_url=IMAGE_URL, crop=((10, 20), (30, 40)), unsafe=False
        )
        expect(url.startswith("unsafe")).to_be_false()
